#include <Wire.h>
#define PWM13       OCR4A

//  SETUP I2C
byte DRV = 0x5A;    //DRV2605 slave address
byte ModeReg = 0x01;
byte TCA = 0x70;

int analogPin = A5;
int val = 0;
int vref = 5;
float res = 1023.0;
int ports[2] = {1, 2};
int delaytime = 3500;

void setup() {
  Wire.begin();
  Serial.begin(9600); 
  pwm13configure();    //sets up PWM signal
  delay(2);
  for (int i = 0; i < sizeof(ports)/sizeof(int); i++)
  {
    TCA9548A(ports[i]);
    initializeDRV2605();
    delay(10);
    pulse(0.1, 10);
  }
}

void loop()
{
    TCA9548A(1);
    // put your main code here, to run repeatedly:
    val = analogRead(A5);
    Serial.println(val / res * vref);
  
    if ((val / res) * vref > 1.25) 
    {
        usdelay(delaytime);
        while ((val / res) * vref > 1.25) 
        {
            TCA9548A(1);
            val = analogRead(A5); 
            if ((val / res) * vref < 1.25) 
            {
                delaytime = 3500;
                break;
            }
            
            pulse(1, 100);
            TCA9548A(2);
            pulse(1, 100);

          
            delaytime = 0;
        }
    }
    
    return 0;
}

 


// Basic functions:

/*
1. void pulse(double intensity, double milliseconds)
pulse drives the hammer towards the closed end of the TacHammer
intensity defines the strength of the pulse. It ranges from [0-1] with 1 being strongest
When the hammer rebounds off of the repelling magnetic array, the inaudible pulse haptic
sensation is created milliseconds is the length of time the coil is activated in ms pulse is
intended to be sequenced with subsequent pulse and hit commands and if called on its own,
the hammer may travel after the rebound and strike the open end 
*/

void pulse(double intensity, double milliseconds) 
{
  int minimumint = 140;
  int maximumint = 255;
  int pwmintensity = (intensity * (maximumint - minimumint)) + minimumint;
  standbyOffB();
  PWM13 = pwmintensity;
  usdelay(milliseconds);
  standbyOnB();
}



/*
4. void pause(double milliseconds)
pause waits a duration of time defined in ms
*/
void pause(double milliseconds) 
{
  double us = milliseconds - ((int)milliseconds);
  standbyOnB();
  for (int i = 0; i <= milliseconds; i++) 
  {
    delay(1);
  }
  delayMicroseconds(us * 1000);
}

/*
5. void usdelay(double time)
usdelay implements a delay of the whole program in ms, it does not the same as pause
Can you describe the difference between the functions?
*/
void usdelay(double time) 
{
  double us = time - ((int)time);
  for (int i = 0; i <= time; i++) 
  {
    delay(1);
  }
  delayMicroseconds(us * 1000);
}






void standbyOnB() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);            // sets register pointer to the mode register (0x01)
  Wire.write(0x43);               // Puts the device pwm mode
  Wire.endTransmission();
}

void standbyOffB() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);               // sets register pointer to the mode register (0x01)
  Wire.write(0x03);               // Sets Waveform Mode to pwm
  Wire.endTransmission();
}

void initializeDRV2605() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);            // sets register pointer to the mode register (0x01)
  Wire.write(0x00);               // clear standby
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x1D);              // sets register pointer to the Libarary Selection register (0x1D)
  Wire.write(0xA8);              // set RTP unsigned
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x03);
  Wire.write(0x02);              // set to Library B, most aggresive
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x17);               // sets full scale reference
  Wire.write(0xff);               //
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);               // sets register pointer to the mode register (0x01)
  Wire.write(0x03);               // Sets Mode to pwm
  Wire.endTransmission();

  delay(100);

}

void TCA9548A(uint8_t bus){
  Wire.beginTransmission(TCA);
  Wire.write(1 << bus);
  Wire.endTransmission();
}
/*
DEAR STUDENTS, PLEASE DO NOT MODIFY THIS SECTION
iF YOU NEED ANY ASSISTANCE, PLEASE NOTIFY THE TA OR THE MAIN LECTURER
*/

// Configure the PWM clock
#define PWM12k  5   //  11719 Hz

void pwm13configure() {
  // TCCR4A configuration
  TCCR4A = 0;

  // TCCR4B configuration
  TCCR4B = PWM12k;

  // TCCR4C configuration
  TCCR4C = 0;

  // TCCR4D configuration
  TCCR4D = 0;

  // TCCR4D configuration
  TCCR4D = 0;

  // PLL Configuration
  // Use 96MHz / 2 = 48MHz
  PLLFRQ = (PLLFRQ & 0xCF) | 0x30;
  // PLLFRQ=(PLLFRQ&0xCF)|0x10; // Will double all frequencies

  // Terminal count for Timer 4 PWM
  OCR4C = 255;

  //pwmSet6();
  pwmSet13();
}


// Set PWM to D13 (Timer4 A)
void pwmSet13() {
  OCR4A = 0;  
  DDRC |= _BV(7); 
  TCCR4A = 0x82; 
}

